-- bmem_dp.vhd
--
-- Lehrbuch Digitaltechnik
--
-- (c) W. Gehrke, M. Winzker, K. Urbanski, R. Woitowitz, 2016

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity bmem_dp is
   generic (
      DW  : integer := 16;   -- Data Width
      AW  : integer := 10 ); -- Address Width

   port (
      -- Port 1
      clk1 : in  std_logic;  -- Clock
      we1  : in  std_logic;  -- Write enable
      a1   : in  std_logic_vector(AW-1 downto 0); -- Address
      d1   : in  std_logic_vector(DW-1 downto 0); -- Data in
      q1   : out std_logic_vector(DW-1 downto 0); -- Data out
 
      -- Port 2 
      clk2 : in  std_logic; -- Clock
      we2  : in  std_logic; -- Write enable
      a2   : in  std_logic_vector(AW-1 downto 0); -- Address
      d2   : in  std_logic_vector(DW-1 downto 0); -- Data in
      q2   : out std_logic_vector(DW-1 downto 0)  -- Data out
      );
end;

architecture rtl of bmem_dp is
   type tmem is array(0 to 2**AW-1) of std_logic_vector(DW-1 downto 0);
   -- Hier wird die "shared variable" definiert
   shared variable mem : tmem := ((others=> (others=>'0')));
   signal q1_sig : std_logic_vector(DW-1 downto 0) := (others=>'0');
   signal q2_sig : std_logic_vector(DW-1 downto 0) := (others=>'0');

begin
   q1 <= q1_sig;
   q2 <= q2_sig;
 
   -- Port 1
   process begin
      wait until rising_edge(clk1);
      if (we1 = '1') then
         mem(to_integer(unsigned(a1))) := d1;
      end if;
      q1_sig <= mem(to_integer(unsigned(a1)));
   end process;
 
   -- Port 2
   process begin
      wait until rising_edge(clk2);
      if (we2 = '1') then
         mem(to_integer(unsigned(a2))) := d2;
      end if;
      q2_sig <= mem(to_integer(unsigned(a2)));
   end process; 
end;
