-- tb.vhd
--
-- Lehrbuch Digitaltechnik
--
-- (c) W. Gehrke, M. Winzker, K. Urbanski, R. Woitowitz, 2016

use std.textio.all; -- bei Benutzung der Standard-Bibliothek 
 -- ist keine Library-Anweisung erforderlich
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;

entity tb is
end;

architecture tb_arch of tb is
   signal bin_val : std_logic_vector(3 downto 0);
   signal sev_seg_code : std_logic_vector(6 downto 0);
begin
   dut : entity work.bin2sevenseg -- DUT: Device Under Test 
      port map (
         bin      => bin_val,
         sevenseg => sev_seg_code );
 
process -- Prozess zum Anlegen von Stimuli und zum Ueberprufen
        -- der Ausgabewerte des device under test (DUT)"
 
   file stimuli_file         : text; -- Filedeskriptoren anlegen
   file resultat_file        : text;
   variable stim_file_status : FILE_OPEN_STATUS; -- Filestatus
   variable res_file_status  : FILE_OPEN_STATUS;
   variable l                : line; -- Variable vom Typ line fuer Text-IO
   variable stim             : std_logic_vector(3 downto 0);
   variable exp              : std_logic_vector(6 downto 0);
   variable wait_time        : time;
   variable errors_detected  : natural := 0;
begin
   -- Dateien ffnen
   file_open(stim_file_status, stimuli_file, "timuli.txt", READ_MODE); 
   file_open(res_file_status, resultat_file, "result.txt", WRITE_MODE);
 
   -- Dateien erfolgreich geffnet?
   if stim_file_status = OPEN_OK and res_file_status = OPEN_OK then 
      while not endfile(stimuli_file) loop -- Dateiende?
         readline(stimuli_file,l); -- Eine Zeile lesen
         read(l,stim); -- Stimuli lesen
         bin_val <= stim;
         read(l,wait_time); -- Wartezeit lesen
         wait for wait_time; -- Warten
         read(l,exp); -- Erwarteten Ausgabewert lesen 
         write (l,stim); -- Stimuli und Ausgabewerte
         write (l,string'(" ")); -- in Resultat-Datei schreiben
         write (l,sev_seg_code);
         write (l,string'(" "));
         assert sev_seg_code = exp 
                 report "Simulation error detected" severity warning;
         if sev_seg_code = exp then -- in Resultat-Datei schreiben
            write (l,string'("Ok"));
         else
            write (l,string'("Error -- Expected: "));
            write (l,exp);
            errors_detected := errors_detected + 1; -- Fehlerzaehler+1
         end if; 
         writeline(resultat_file,l);
      end loop;
      -- Am Ende der Simulation den Fehlerzaehler ausgeben
      write (l,string'("--------------------------------"));
      writeline(resultat_file,l);
      write (l,string'("Total Error Count: "));
      write (l,errors_detected);
      writeline(resultat_file,l);
      write (l,string'("--------------------------------"));
      writeline(resultat_file,l);
      file_close(stimuli_file); -- Dateien schliessen 
      file_close(resultat_file);
   end if;
   -- Simulation mit Assert-Anweisung beenden
   assert false report "Simulation finished." severity failure;
end process; 
end;
